@import "css/theme.css";

@font-face {
    font-family: 'DistrictThinRegular';
    src: url('distth___-webfont.eot');
    src: url('distth___-webfont.eot?iefix') format('eot'),
         url('distth___-webfont.ttf') format('truetype');
    font-weight: normal;
    font-style: normal;
}

body { color: #3a3a3a; }

body, th, tr {
    font-family: arial, verdana, sans, geneva;
    font-size: 17px;
}

.wy-nav-content-wrap { background-color: #f8f8f8; }

.wy-nav-top {
    background-color: #e3e3e3;
    color: #3f7516;
}
.wy-nav-top a {
    font: bold 30px "DistrictThinRegular";
    color: #3f7516;
}

.wy-side-nav-search { background-color: #e3e3e3; }
.wy-side-nav-search input[type="text"] { border-color: #3f7516; }
.wy-side-nav-search, .wy-side-nav-search > a, .wy-side-nav-search img { padding-top: 0; }

.wy-menu-vertical li.current > a { background-color: white }

.wy-nav-content div[aria-label="breadcrumbs navigation"] { display: None }

.wy-nav-content {
    max-width: 960px;
    background-color: white;
    margin: 0;
}

p {
    margin-bottom: 15px;
    text-align: justify;
    font-size: 17px;
}

em { font-style: italic }

blockquote { margin: 0 0 15px 0 }

.htmlscreen { margin-bottom: 15px }

h1 {
    color: #3a3a3a;
    border: 0;
    font: bold 34px "DistrictThinRegular";
}

h2 {
    margin: 25px 0 15px 0;
    color: #3f7516;
    border-bottom: 1px solid #9f9f9f;
    font-size: 26px;
}

h3 {
    margin: 20px 0 15px 0;
    font-size: 23px;
}

h4 {
    margin: 20px 0 15px 0;
    font-style: italic;
    font-size: 20px;
}

h2, h3, h4, .rst-content .toctree-wrapper p.caption {
    font-family: Arial, Verdana, 'Bitstream Vera Sans', Helvetica, sans-serif;
    font-weight: normal;
}

.rst-content .section ul li { list-style-type: square }
.rst-content .section ol.arabic li ul li { list-style-type: square }
.rst-content ol.arabic li { margin-left: 40px }

.rst-content a {
    color: #3f7516;
    border-bottom: 1px dotted #3f7516;
}

.rst-content blockquote { margin: 0; }
.rst-content .highlight > pre, .rst-content .linenodiv > pre { line-height: 1.5 }

div[class^="highlight"] {
    margin: 0 0 15px 24px;
    background-color: #f8f8f8;
    background-repeat: no-repeat;
    border: 0px;
    border-left-width: 2px;
    border-left-style: solid;
}

div[class^="highlight"] div.highlight {
    background: None;
    border: 0;
    margin: 0;
    padding: 0;
}

div[class^="highlight"] pre { font-size: 14px }

.note div[class^="highlight"] { background: none; }

.wy-table-responsive table td { white-space: normal; }

.rst-content table.docutils {
    width: 100%;
    border: 1px solid #3f7516;
    border-collapse: collapse;
}

.rst-content table.docutils:not(.field-list) tr:nth-child(2n-1) td { background: none }
.rst-content table.docutils:not(.field-list) tr:nth-child(2n) td { background: #edfae1 }
.rst-content table.docutils thead th { background: #d4e2c8; }

div.description-left { float: left; }
div.description-right {
    padding-left: 20px;
    overflow: hidden;
}

@media print {
    .wy-nav-content { max-width: none }
    body, th, tr { font-size: 14px; }
    p {
        font-size: 14px;
        margin-bottom: 6px;
        line-height: normal
    }
    h1 { font-size: 28px }
    h2 { font-size: 22px; }
    h3 { font-size: 20; }
    h4 { font-size: 17px; }
    div[class^="highlight"] pre { font-size: 13px }
}


